{% extends "layout.html" %}
{%- block htmltitle -%}
<title>{{ shorttitle }}</title>
{%- endblock -%}
{% block body %}
  <h1>{{ docstitle|e }}</h1>
  <p>
  {% trans %}Welcome! This is the documentation for Python {{ release }}.{% endtrans %}
  </p>
  <p><strong>{% trans %}Parts of the documentation:{% endtrans %}</strong></p>
  <table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("whatsnew/" + version) }}">{% trans %}What's new in Python {{ version }}?{% endtrans %}</a><br/>
        <span class="linkdescr"> {% trans whatsnew_index=pathto("whatsnew/index") %}or <a href="{{ whatsnew_index }}">all "What's new" documents</a> since 2.0{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("tutorial/index") }}">{% trans %}Tutorial{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}start here{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("library/index") }}">{% trans %}Library Reference{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}keep this under your pillow{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("reference/index") }}">{% trans %}Language Reference{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}describes syntax and language elements{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("using/index") }}">{% trans %}Python Setup and Usage{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}how to use Python on different platforms{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("howto/index") }}">{% trans %}Python HOWTOs{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}in-depth documents on specific topics{% endtrans %}</span></p>
    </td><td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("installing/index") }}">{% trans %}Installing Python Modules{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}installing from the Python Package Index &amp; other sources{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("distributing/index") }}">{% trans %}Distributing Python Modules{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}publishing modules for installation by others{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("extending/index") }}">{% trans %}Extending and Embedding{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}tutorial for C/C++ programmers{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("c-api/index") }}">{% trans %}Python/C API{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}reference for C/C++ programmers{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("faq/index") }}">{% trans %}FAQs{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}frequently asked questions (with answers!){% endtrans %}</span></p>
    </td></tr>
  </table>

  <p><strong>{% trans %}Indices and tables:{% endtrans %}</strong></p>
  <table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("py-modindex") }}">{% trans %}Global Module Index{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}quick access to all modules{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("genindex") }}">{% trans %}General Index{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}all functions, classes, terms{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("glossary") }}">{% trans %}Glossary{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}the most important terms explained{% endtrans %}</span></p>
    </td><td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("search") }}">{% trans %}Search page{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}search this documentation{% endtrans %}</span></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("contents") }}">{% trans %}Complete Table of Contents{% endtrans %}</a><br/>
         <span class="linkdescr">{% trans %}lists all sections and subsections{% endtrans %}</span></p>
    </td></tr>
  </table>

  <p><strong>{% trans %}Meta information:{% endtrans %}</strong></p>
  <table class="contentstable" align="center"><tr>
    <td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("bugs") }}">{% trans %}Reporting bugs{% endtrans %}</a></p>
      <p class="biglink"><a class="biglink" href="https://devguide.python.org/docquality/#helping-with-documentation">{% trans %}Contributing to Docs{% endtrans %}</a></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("about") }}">{% trans %}About the documentation{% endtrans %}</a></p>
    </td><td width="50%">
      <p class="biglink"><a class="biglink" href="{{ pathto("license") }}">{% trans %}History and License of Python{% endtrans %}</a></p>
      <p class="biglink"><a class="biglink" href="{{ pathto("copyright") }}">{% trans %}Copyright{% endtrans %}</a></p>
    </td></tr>
  </table>
{% endblock %}
